#!/usr/bin/bash

export LANG=en_US.UTF-8

trap ctrl_c INT

function ctrl_c() {
    echo "*** Trapped CTRL-C ***"
    exit 1
}

_stage=$1
_stage_lock="/root/initial/tasks/${_stage}.done"
_centos_release=$(/bin/rpm --eval='%centos_ver')

stage1() {
    check_stage
    while ! which puppet &> /dev/null; do
        echo "puppet not installed"
        dnf -y install nf-puppet3
        sleep 5
    done
    sed -i "s#ETHDEV.*#ETHDEV $(ip route show default 0.0.0.0/0 | grep -Po '(?<=dev )\S+')#" /etc/wwwacct.conf
    timedatectl set-timezone "${TIME_ZONE}"
    timedatectl set-local-rtc 0
    timedatectl set-ntp true
    mkdir -vp /opt/eig_linux/etc/puppetfeatures
    touch /opt/eig_linux/etc/puppetfeatures/{datacycle,remi_repo}
    FACTER_foreman_organization=${BRAND} FACTER_foreman_location=${LOCATION} timeout -s 9 1m puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration
    while [[ $? =~ ^(1|137)$ ]]; do
        sleep 5
        FACTER_foreman_organization=${BRAND} FACTER_foreman_location=${LOCATION} puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration
    done
    puppet agent --disable 'provisioning in progress'
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage2() {
    check_stage
    puppet agent --enable

    ## applying repos,puppet config
    timeout -s 9 1m puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    while [[ $? =~ ^(1|137)$ ]]; do
        sleep 5
         timeout -s 9 1m puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    done

    ## this will normally fail if prov process failed to configure server in correct hostgroup.
    timeout -s 9 1m puppet agent -t --tags=yum_repos,puppet_agent,resolvers
    while [[ $? =~ ^(1|137)$ ]]; do
        sleep 5
         timeout -s 9 1m puppet agent -t --tags=yum_repos,puppet_agent,resolvers
    done

    puppet agent --disable 'provisioning in progress'
    rpm --import https://rpms.remirepo.net/enterprise/9/RPM-GPG-KEY-remi
    dnf clean all -y --enablerepo=*
    dnf makecache
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage3() {
    check_stage
    service ipaliases start
    if [[ ! -z ${cloud_name+x} ]]; then
      /usr/local/cpanel/scripts/build_cpnat
      md5sum /root/.accesshash | awk {'print $1'} | tr -d '\n' > /root/.accesshash.md5
      echo SSD > /opt/eig_linux/etc/puppetfeatures/disk_type
    fi
    puppet agent --enable

    if ! systemctl is-active mysql &> /dev/null; then
      if ! systemctl start mysql &> /dev/null; then
        echo "MySQL failed to start"
        exit 1
      fi
    fi

    timeout -s 9 600s puppet agent -t
    while [ $? -ne 0 ]; do
        if ! systemctl is-active mysql &> /dev/null; then
          if ! systemctl start mysql &> /dev/null; then
            echo "MySQL failed to start"
            exit 1
          fi
        fi
        sleep 5
        timeout -s 9 600s puppet agent -t
    done
    if ! systemctl is-active mysql &> /dev/null; then
      if ! systemctl start mysql &> /dev/null; then
        echo "MySQL failed to start"
        exit 1
      fi
    fi

#    /opt/hgmods/multi_shared.pl TODO

    if [[ -z ${prov_env+x} ]]; then
      ### custapi randomly drops requests causing it to fail. Adding this as a work around
      /opt/hgmods/hal_accesshash.pl
      while [ $? -eq 1 ]; do
        sleep 5
        /opt/hgmods/hal_accesshash.pl
      done
    fi
    sed -i -r '/\/(home[1-9]?)/s/[0-9]$/0/' /etc/fstab
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage4() {
    check_stage
    timeout -s 9 1m puppet agent -t
    while [ $? -ne 0 ]; do
        if ! systemctl is-active mysql &> /dev/null; then
          if ! systemctl start mysql &> /dev/null; then
            echo "MySQL failed to start"
            exit 1
          fi
        fi
        sleep 5
        timeout -s 9 1m puppet agent -t
    done
    if ! systemctl is-active mysql &> /dev/null; then
      if ! systemctl start mysql &> /dev/null; then
        echo "MySQL failed to start"
        exit 1
      fi
    fi
    if ! mysql -BNe 'select User from user where User = "collectd"' -Dmysql | grep -q collectd; then
      _passwd=$(awk '/Password/ {print $2}' /etc/collectd.conf| sed 's/"//g')
      while ! mysql -BNe "CREATE USER 'collectd'@'localhost' IDENTIFIED BY '${_passwd}'; GRANT USAGE, PROCESS, REPLICATION CLIENT ON *.* TO collectd@localhost;"; do
        echo "${HOSTNAME} fail to add grant"
        sleep 5
      done
      echo "${HOSTNAME} added grant"
    fi

    if rpm -q --quiet python3-perf &> /dev/null; then
      dnf remove -y python3-perf
    fi
    while ! dnf -y update; do
      echo "Pending DNF updates"
      sleep 5
      dnf -y update
    done

    /scripts/rebuildhttpdconf
    while ! httpd -t; do
      sleep 5
      /scripts/rebuildhttpdconf
    done

    /usr/share/mod_lsapi/scriptlets/mod_lsapi_post.sh

    if ! rpm -q --quiet tuned &> /dev/null; then
      dnf install -y tuned
      systemctl --now enable tuned
      tuned-adm profile virtual-guest
    fi

    if [[ -s /etc/motd ]]; then
      > /etc/motd
    fi

    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

stage5() {
    check_stage

    ### right now in OCI when kernel updates via cloud-init default kernel entry is not getting updated correctly
    nf_kernel=$(\ls /boot/vmlinuz-5.14.0-*nf* | tail -1)
    if [[ ${nf_kernel} != $(grubby --default-kernel) ]]; then
      echo "Setting default kernel to ${nf_kernel}"
      grubby --set-default=${nf_kernel}
    fi

    if _var=$(rpm -qa kernel-core | grep -v nf | paste -sd " " | grep [a-z]); then
      if [[ $(uname -r) =~ nf ]]; then
        dnf remove -y ${_var}
      else
        echo "Can't remove extra kernel-core RPMs without first being booted into NF kernel"
        exit 1
      fi
    fi

    dnf -y remove initial-scripts-gt-${BRAND}
    if [[ "$(dmidecode -s chassis-asset-tag)" != 'OracleCloud.com' ]]; then
      echo "Rebooting in 2 Minutes"
      systemctl --now enable atd
      echo "shutdown -rf now" | at now + 2 min
    fi
    echo "$(date) :: Completed ${_stage}"
    touch "${_stage_lock}"
}

check_stage() {
    if [ -f "${_stage_lock}" ]; then
        exit 0
    fi
    echo "$(date) :: Running ${_stage}"
}

${_stage}
